import numpy as np


def readArray(filename):
    '''
    lees een door saveArray opgeslagen array weer uit

    arugmenten:
        filename: filenaam waar de data staat
    '''
    data = np.loadtxt(filename, unpack=False, delimiter=',')
    return data


if __name__ == '__main__':
    # module wordt uitgevoerd
    import matplotlib.pyplot as plt

    # data van 1 kanaal weergeven
    fn = 'meting_test1_1603898691.txt'
    data = readArray(fn)
    time = data[:, 0]
    plt.plot(time, data[:, 1])
    plt.show()



#!/usr/bin/env python

from sympy import pi, cos, sin, sqrt, arg, exp, Abs, S
from sympy import integrate, Rational, nfloat, conjugate, Symbol
from sympy.functions.special.delta_functions import Piecewise

t = Symbol('t')


def Heaviside(arg):
    ''' Heaviside die ook op arg=0 gedefinieerd is '''
    return Piecewise((0, arg < 0), (1, arg >= 0))


# functies om de fourier-reeks coefficienten uit te rekenen
def an(f, T, n, omega0):
    return 2/T*integrate(f*cos(n*omega0*t), (t, 0, T))


def bn(f, T, n, omega0):
    return 2/T*integrate(f*sin(n*omega0*t), (t, 0, T))


def a0(f, T, omega0):
    return 1/T*integrate(f, (t, 0, T)).nsimplify()


def fourreeks(f, T, N):
    N += 1  # voor Maple
    a = (N-1)*[0]
    b = (N-1)*[0]
    omega0 = 2*pi/T
    som = a0(f, T, omega0)
    for n in range(1, N):
        a[n-1] = an(f, T, n, omega0)
        b[n-1] = bn(f, T, n, omega0)
        som += (a[n-1]*cos(n*omega0*t) + b[n-1]*sin(n*omega0*t)).nsimplify()
    return som, a, b


def spec(a, b, T):
    omega0 = 2*pi/T
    mag = [nfloat(sqrt(a[i]**2 + b[i]**2)) for i in range(len(a))]
    phi = len(a)*[0]
    for i in range(len(b)):
        if a[i] == S.Zero:
            phi[i] = 0
        else:
            phi[i] = nfloat(arg(a[i] - S.ImaginaryUnit*b[i]))
    mag.insert(0, a0(f, T, omega0))
    if mag[0] >= 0:
        phi.insert(0, 0)
    else:
        phi.insert(0, pi)
    return (mag, phi)


def periodiek(f, T, N):
    som = S(0)
    for n in range(N):
        som += (Heaviside(t-n*T) - Heaviside(t-(n+1)*T))*f.replace(t, t-n*T)
    return som


def plot_color(p):
    color = ['r', 'b', 'g', 'y', 'm']
    Nc = len(color)
    for i, pc in enumerate(p):
        pc.line_color = color[i % Nc]


if __name__ == '__main__':
    from sympy.plotting import plot
    import matplotlib.pyplot as plt

    f = Heaviside(t+1) - Heaviside(t-1)
    T = 4
    N = 5
    teind = 10
    som, a, b = fourreeks(f, T, N)
    mag, phase = spec(a, b, T)
    plot(som, (t, 0, teind))
    p = plot(periodiek(f, T, int(teind/T)+1), som, (t, 0, teind),
             adaptive=False, show=False)
    plot_color(p)
    p.show()
    phase, mag = spec(a, b, T)
    plt.figure()
    plt.subplot(2, 1, 1)
    plt.plot(range(len(phase)), phase)
    plt.subplot(2, 1, 2)
    plt.plot(range(len(mag)), mag)
    plt.show()



#!/usr/bin/env python3

import matplotlib.ticker as mticker
"""
(c) 2018, D.D.Land <d.d.land@hhs.nl>

Module om figuren weer te geven zoals aangegeven in het Rapportage document
van de afdeling Technische Natuurkunde van de Haagse Hogeschool.

versie: 1.1

20190323: docstrings aangepast aan eisen programmeren.
20180317: herschreven naar class structuur om meerdere (sub)figuren hun eigen
aantal digits op de assen te kunnen geven.
"""


class TNFormatter(mticker.Formatter):
    """
    Class om een vast aantal significante getallen weer te geven in de
    assen van een figuur.

    Class wordt aangeroepen vanuit de axis formatter van een matplotlibfiguur.
    """

    def __init__(self, length=3):
        """
        Initialisatie van de TNFormatter class. Het aantal significante
        getallen (length) wordt gebruikt in andere functies om de getallen
        juist op te maken.

        argumenten:
            lengte: Geeft aan hoeveel significante getallen weergegeven
                    moeten worden.
        return:
            none
        """
        self.length = length

    def __call__(self, x, pos=None):
        """
        Functie die vanuit matplotlib aangeroepen wordt als een figuur
        gemaakt (of bekeken) wordt.

        argumenten:
            x: getal waarvan de opmaak aangepast moet worden
            pos: niet gebruikt argument vanuit matplotlib.
        return:
            string met daarin het juist opgemaakte getal.

        Herschrijft de getallen op de assen naar getallen met een vast aantal
        digits.
        """
        s = '%.10e' % x
        tup = s.split('e')
        signif = tup[0].rstrip('.')
        neg = ''
        if signif[0] == '-':
            neg = '-'
            signif = signif[1:]
        sign = tup[1][0].replace('+', '')  # verwijder +
        expo = tup[1][1:].lstrip('0')
        if expo:  # getal groter dan 10
            if self.length <= abs(int(expo)):  # schrijf als 10 macht
                expon = '10^{%s%s}' % (sign, expo)
                if self.length == 1:  # allen getal voor de . houden...
                    splt = signif.split('.')
                    signif = splt[0]
                else:  # meer dan 1 cijfer significant
                    signif = signif[:self.length+1]
                    if len(signif) < self.length+1:
                        signif += '0' * (self.length + 1 - len(signif))
                s = r'%s%s{\cdot}%s' % (neg, signif, expon)
            else:  # schrijf als getal
                s = '%.10f' % x
                extra0 = 0
                if s[0] == '-':
                    s = s[1:]
                if s[0] == '0':  # kleiner dan 1
                    splt = s.split('.')
                    zeros = splt[1].lstrip('0')
                    num = len(splt[1]) - len(zeros)
                    s = r'%s%s%s' % (neg, s[:self.length+2], '0'*num)
                else:
                    s = r'%s%s' % (neg, s[:self.length+1])
        else:  # exponent is 0
            if len(signif) >= self.length:
                if self.length == 1:
                    s = r'%s%s' % (neg, signif[:self.length])
                else:
                    s = r'%s%s' % (neg, signif[:self.length+1])
            else:
                s = r'%s%s%s' % (neg, signif, '0'*(self.length-len(signif)))
        if s[-1] == '.':  # laatste teken een ., mag dus weg.
            s = s[:-1]
        # vervang punten door komma's.
        # Extra witruimte rond komma met {} uit string weggehaald.
        return "${}$".format(s.replace('.', '{,}'))

    def legenda(self, x, pos=None):
        """
        Functie aangeroepen vanuit matplotlib. Functie geeft aanroep naar
        __call__ terug.

        argumenten:
            x: getal waarvan de opmaak aangepast moet worden
            pos: niet gebruikt argument vanuit matplotlib.

        """
        return self.__call__(x, pos)


def label_x(grootheid, eenheid, ax, haak='[]', text=''):
    """
    Zet label van de as op een (relatief) makkelijke manier.

    argumenten:
        grootheid: grootheid van de waarden op de as.
        eenheid: eenheid van de waarden op de as.
        ax: matplotlib ax object waarin de x-as zicht bevindt.
        haak: standaard [], anders string met linker haak, rechter haak.
        text: eventuele tekst die voor de grootheid weergegeven moet worden.
    return:
        none
    """
    ax.xaxis.set_label_text(r'%s $\,%s \, %s\mathrm{%s}%s$' % (text,
                            grootheid, haak[0], eenheid, haak[1]))


def label_y(grootheid, eenheid, ax, haak='[]', text=''):
    """
    Zet label van de as op een (relatief) makkelijke manier.

    argumenten:
        grootheid: grootheid van de waarden op de as.
        eenheid: eenheid van de waarden op de as.
        ax: matplotlib ax object waarin de x-as zicht bevindt.
        haak: standaard [], anders string met linker haak, rechter haak.
        text: eventuele tekst die voor de grootheid weergegeven moet worden.
    return:
        none
    """
    ax.yaxis.set_label_text(r'%s $\,%s \, %s\mathrm{%s}%s$' % (text,
                            grootheid, haak[0], eenheid, haak[1]))


# Backwards compatiblity (fix_axis functie aanroep)

# Variabelen die na de import door de gebruiker aangepast kunnen worden.
# Variabelen geven het aantal significante getallen voor de x-, y-as.

PRECISION_X = 2
PRECISION_Y = 2


def fix_axis(ax):
    """
    Functie die de nieuwe Class implementatie beschikbaar houdt
    voor de oude functie definities.

    argumenten:
        ax: matplotlib ax object

    module argumenten:
        PRECISION_X: aantal significante getallen op de x-as
        PRECISION_Y: aantal significante getallen op de x-as
    """
    ax.yaxis.set_major_formatter(TNFormatter(length=PRECISION_Y))
    ax.xaxis.set_major_formatter(TNFormatter(length=PRECISION_X))



import time
import os
import numpy as np

"""
Module waarin voor verschillende DACs code staat om de DAC uit te lezen.
De functies geven een array terug met data (voltage), standaard 1000
meetpunten, zo snel mogelijk na elkaar gemeten.


    functies:
        readMCP3008: lees data van een MSP3008
                   https://learn.adafruit.com/mcp3008-spi-adc/python-circuitpython
        readADS:   lees data van de ADS1015 en ADS1115 van adafruit.
                   https://learn.adafruit.com/adafruit-4-channel-adc-breakouts/
        readADCpi: lees data van de ADC Pi Plus
                   https://www.abelectronics.co.uk/kb/article/1044/adc-pi
        readPiPlate: lees analoge data van de PiPlate: DAQCplate en DAQ2plate
                   https://pi-plates.com
        readArduino: lees data van een arduino
                   (met standaard AnalogReadSerial.ino geladen)

"""


def readMCP3008(channel, N=1000, t0=0):
    """
    lees de MCP3008 op kanaal=channel uit voor N samples

    argumenten:
        channel: AnalogIn object met het juiste kanaal of een lijst van
                 objecten voor meerdere kanalen.
        N: aantal samples om te lezen, standaard 10000
        t0: begintijd. Als t=0 begint de tijd hiermee, anders met de opgegeven
           waarde.

    return:
        data: 2d of n-d numpy array met N data- en tijd samples.
    """
    return readADS(channel, N, t0)


def readADS(channel, N=1000, t0=0):
    """
    lees de ADS op kanaal=channel uit voor N samples

    argumenten:
        channel: AnalogIn object met het juiste kanaal of een lijst van
                 objecten voor meerdere kanalen.
        N: aantal samples om te lezen, standaard 10000
        t0: begintijd. Als t=0 begint de tijd hiermee, anders met de opgegeven
           waarde.

    return:
        data: 2d of n-d numpy array met N data- en tijd samples.
    """

    if np.size(channel) == 1:
        data = np.zeros((2, N))
        if t0 == 0:
            t0 = time.time()
        for i in range(N):
            data[:, i] = (time.time() - t0, channel.voltage)
    else:
        channels = len(channel)
        data = np.zeros((channels+1, N))
        samples = np.zeros(channels)
        if t0 == 0:
            t0 = time.time()
        for i in range(N):
            for sample in range(channels):
                samples[sample] = channel[sample].voltage
            data[:, i] = time.time()-t0, *samples
    return data


def readADCpi(adc, channel, N=1000, t0=0):
    """
    lees de ADCpi op kanaal=channel uit voor N samples

    argumenten:
        adc: ADCpi object
        channel: kanaal waarop de sensor aangesloten is (of lijst met
                 meerdere kanelen)
        N: aantal samples om te lezen, standaard 10000
        t0: begintijd. Als t=0 begint de tijd hiermee, anders met de opgegeven
           waarde.

    return:
        data: 2d of n-d numpy array met N data- en tijd samples.
    """

    if np.size(channel) == 1:
        data = np.zeros((2, N))
        if t0 == 0:
            t0 = time.time()
        for i in range(N):
            data[:, i] = (time.time() - t0, adc.read_voltage(channel))
    else:
        channels = len(channel)
        data = np.zeros((channels+1, N))
        samples = np.zeros(channels)
        if t0 == 0:
            t0 = time.time()
        for i in range(N):
            for sample in range(channels):
                samples[sample] = adc.read_voltage(channel[sample])
            data[:, i] = time.time()-t0, *samples
    return data


def readPiPlate(plate, channel, N=1000, t0=0, ADC=0):
    """
    lees de PiPlate op kanaal=channel uit voor N samples

    argumenten:
        adc: PiPlate (DAQ1 of DAQ2) object
        channel: kanaal waarop de sensor aangesloten is (of lijst met
                 meerdere kanelen)
        N: aantal samples om te lezen, standaard 10000
        t0: begintijd. Als t=0 begint de tijd hiermee, anders met de opgegeven
           waarde.
        ADC: 0 voor eerste DAQ, ander getal indien meerdere DAQ

    return:
        data: 2d of n-d numpy array met N data- en tijd samples.
    """

    if np.size(channel) == 1:
        data = np.zeros((2, N))
        if t0 == 0:
            t0 = time.time()
        for i in range(N):
            data[:, i] = (time.time() - t0, plate.getADC(ADC, channel))
    else:
        channels = len(channel)
        data = np.zeros((channels+1, N))
        samples = np.zeros(channels)
        if t0 == 0:
            t0 = time.time()
        for i in range(N):
            for sample in range(channels):
                samples[sample] = plate.getADC(ADC, channel[sample])
            data[:, i] = time.time()-t0, *samples
    return data


def readArduino(ser, N=1000, t0=0, Ndata=1, seperator=';'):
    """
    Lees de arduino voor N datapunten uit.

    argumenten:
       ser: serial object
        N: aantal samples om te lezen, standaard 10000
        t0: begintijd. Als t=0 begint de tijd hiermee, anders met de
           opgegeven waarde.
        Ndata: aantal sensorvalues in de arduino stream.
        seperator: scheidingstekens voor het uitlezen van meerdere sensor
           waarden uit de stream. Standaard de :

    return:
        data: 2d of n-d numpy array met N data- en tijd samples.
    """

    data = np.zeros((Ndata+1, N))
    if Ndata == 1:
        if t0 == 0:
            t0 = time.time()
        for i in range(N):
            try:    # mocht de data niet (goed) aankomen
                    # ga dan verder met het volgende datapunt
                data[:, i] = (time.time() - t0,
                              float(ser.readline().decode().strip('\r\n')))
            except Exception:
                pass
    else:
        samples = np.zeros(Ndata)
        if t0 == 0:
            t0 = time.time()
        for i in range(N):
            try:    # mocht de data niet (goed) aankomen
                    # ga dan verder met het volgende datapunt
                ardata = ser.readline().decode().strip('\r\n')
                ardata = ardata.split(seperator)
                ardata = [float(i) for i in ardata]
                data[:, i] = (time.time() - t0, *ardata)
            except Exception:
                pass
    return data



#!/usr/bin/env python

from sympy import pi, cos, sin, sqrt, arg, exp, Abs, S
from sympy import integrate, Rational, nfloat, conjugate, Symbol
from sympy.functions.special.delta_functions import Heaviside
from sympy.plotting import plot
import matplotlib.pyplot as plt

t = Symbol('t', real=True)  # anders complexe tijd!


def ac(f, T, n, omega0):
    return 1/T*integrate(f*exp(-S.ImaginaryUnit*n*omega0*t),
                         (t, -T/2, T/2)).nsimplify()


def fourreeks(f, T, N):
    N += 1  # voor Maple
    a = (N-1)*[0]
    b = (N-1)*[0]
    omega0 = 2*pi/T
    som = a0(f, T, omega0)
    for n in range(1, N):
        a[n-1] = an(f, T, n, omega0)
        b[n-1] = bn(f, T, n, omega0)
        som += (a[n-1]*cos(n*omega0*t) + b[n-1]*sin(n*omega0*t)).nsimplify()
    return som, a, b


def cfourreeks(f, T, N):
    ''' Let op! Periode tussen -T/2 en T/2!  '''
    N += 1
    omega0 = 2*pi/T
    cp = (N-1)*[0]
    cn = (N-1)*[0]
    mag = (2*N-1)*[0]
    phase = (2*N-1)*[0]
    c0 = 1/T*integrate(f, (t, -T/2, T/2)).nsimplify()
    mag[N-1] = Abs(nfloat(c0))
    phase[N-1] = 0
    som = c0
    for n in range(1, N):
        cp[n-1] = ac(f, T, n, omega0)
        cn[n-1] = conjugate(cp[n-1])
        mag[N-n-1] = 2*Abs(nfloat(cn[n-1]))
        mag[N+n-1] = 2*Abs(nfloat(cp[n-1]))
        phase[N-n-1] = arg(cn[n-1])
        phase[N+n-1] = arg(cp[n-1])
        som += (cp[n-1]*exp(S.ImaginaryUnit*n*omega0*t) +
                cn[n-1]*exp(-S.ImaginaryUnit*n*omega0*t))
    cp.insert(0, c0)
    return som, mag, phase


def periodiek(f, T, N):
    som = S(0)
    for n in range(N):
        som += (Heaviside(t-n*T) - Heaviside(t-(n+1)*T))*f.replace(t, t-n*T)
    return som


if __name__ == '__main__':
    f = (Heaviside(t+pi) - Heaviside(t))*(t+pi)
    T = 2*pi
    N = 5
    fr, mag, phase = cfourreeks(f, T, N)
    plot(f, (t, -2*pi, 2*pi), adaptive=False)
    plt.show()
    plt.figure()
    plt.subplot(2, 1, 1)
    plt.stem(range(-N, N+1), mag, 'r')
    plt.subplot(2, 1, 2)
    plt.stem(range(-N, N+1), phase, 'b')
    plt.show()



import pathlib
import setuptools

HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()

setuptools.setup(
    name='TN_code',
    version='1.1',
    description='Code gebruikt bij de opleiding TN van de Haagse Hogeschool',
    long_description=README,
    long_description_content_type="text/markdown",
    url='https://hhs-tn.github.io',
    license='MIT',
    author='Derek Land',
    author_email='d.d.land@hhs.nl',
    packages=setuptools.find_packages(exclude=('voorbeelden'),),
    install_requires=['numpy',
                      'matplotlib',
                      'pandas',
                      'sympy',
                      ],
    classifiers=['License :: OSI Approved :: MIT License',
                 'Natural Language :: Dutch',
                 'Programming Language :: Python :: 3',
                 'Topic :: Utilities',
                 ],
)



import numpy as np


def saveArray(data, filename):
    '''
    slaat de data in een comma gescheiden bestand op.

    argumenten:
        data: 2d numpy array met data
        filename: string met de filenaam
    '''

    np.savetxt(filename, np.column_stack(data), delimiter=',')
